@@ -22,7 +22,9 @@ mono_alc_init (MonoAssemblyLoadContext *alc, MonoDomain *domain)
 	alc->domain = domain;
 	alc->loaded_images = li;
 	alc->loaded_assemblies = NULL;
+	alc->pinvoke_scopes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	mono_coop_mutex_init (&alc->assemblies_lock);
+	mono_coop_mutex_init (&alc->pinvoke_lock);
 }
 
 void
@@ -53,7 +55,9 @@ mono_alc_cleanup (MonoAssemblyLoadContext *alc)
 	 */
 
 	alc->loaded_assemblies = NULL;
+	g_hash_table_destroy (alc->pinvoke_scopes);
 	mono_coop_mutex_destroy (&alc->assemblies_lock);
+	mono_coop_mutex_destroy (&alc->pinvoke_lock);
 
 	mono_loaded_images_free (alc->loaded_images);
 
@@ -106,37 +110,6 @@ ves_icall_System_Runtime_Loader_AssemblyLoadContext_GetLoadContextForAssembly (M
 	return GUINT_TO_POINTER (alc->gchandle);
 }
 
-gpointer
-ves_icall_System_Runtime_Loader_AssemblyLoadContext_InternalLoadUnmanagedDllFromPath (MonoStringHandle fname, MonoError *error)
-{
-	gpointer res = NULL;
-	MonoDl *lib = NULL;
-	char *filename = NULL;
-	char *local_error = NULL;
-
-	g_assert (!MONO_HANDLE_IS_NULL (fname)); // should have already been checked in managed, so we assert
-
-	filename = mono_string_handle_to_utf8 (fname, error);
-	goto_if_nok (error, exit);
-
-	g_assert (g_path_is_absolute (filename)); // again, checked in managed
-
-	lib = mono_dl_open (filename, MONO_DL_LAZY, &local_error);
-
-	if (lib == NULL) {
-		mono_error_set_file_not_found (error, filename, "%s", local_error);
-		goto exit;
-	}
-
-	res = lib->handle;
-
-exit:
-	g_free (lib);
-	g_free (filename);
-	g_free (local_error);
-	return res;
-}
-
 gboolean
 mono_alc_is_default (MonoAssemblyLoadContext *alc)
 {
@@ -160,7 +133,8 @@ invoke_resolve_method (MonoMethod *resolve_method, MonoAssemblyLoadContext *alc,
 	goto_if_nok (error, leave);
 
 	MonoReflectionAssemblyHandle assm;
-	gpointer gchandle = GUINT_TO_POINTER (alc->gchandle);
+	gpointer gchandle;
+	gchandle = GUINT_TO_POINTER (alc->gchandle);
 	gpointer args [2];
 	args [0] = &gchandle;
 	args [1] = MONO_HANDLE_RAW (aname_obj);
