@@ -268,6 +268,10 @@ typedef struct {
 	 * tracking the provenances of objects.
 	 */
 	gpointer (*get_provenance_func) (void);
+	/*
+	 * Same as thread_mark_func, mark the intepreter frames.
+	 */
+	void (*interp_mark_func) (gpointer thread_info, GcScanFunc func, gpointer gc_data, gboolean precise);
 } MonoGCCallbacks;
 
 /* Set the callback functions callable by the GC */
@@ -313,6 +317,14 @@ int mono_gc_get_los_limit (void);
 
 guint64 mono_gc_get_allocated_bytes_for_current_thread (void);
 
+guint64 mono_gc_get_total_allocated_bytes (MonoBoolean precise);
+
+void mono_gc_get_gcmemoryinfo (gint64* fragmented_bytes,
+						       gint64* heap_size_bytes,
+						       gint64* high_memory_load_threshold_bytes,
+						       gint64* memory_load_bytes,
+ 						       gint64* total_available_memory_bytes);
+
 guint8* mono_gc_get_card_table (int *shift_bits, gpointer *card_mask);
 guint8* mono_gc_get_target_card_table (int *shift_bits, target_mgreg_t *card_mask);
 gboolean mono_gc_card_table_nursery_check (void);
