@@ -298,8 +298,6 @@ struct _MonoTableInfo {
 
 #define REFERENCE_MISSING ((gpointer) -1)
 
-typedef struct _MonoDllMap MonoDllMap;
-
 typedef struct {
 	gboolean (*match) (MonoImage*);
 	gboolean (*load_pe_data) (MonoImage*);
@@ -535,8 +533,10 @@ struct _MonoImage {
 	 */
 	void *user_info;
 
+#ifndef DISABLE_DLLMAP
 	/* dll map entries */
 	MonoDllMap *dll_map;
+#endif
 
 	/* interfaces IDs from this image */
 	/* protected by the classes lock */
@@ -562,11 +562,10 @@ struct _MonoImage {
 	MonoConcurrentHashTable *var_gparam_cache;
 	MonoConcurrentHashTable *mvar_gparam_cache;
 
+#ifndef ENABLE_NETCORE
 	/* Maps malloc-ed char* pinvoke scope -> MonoDl* */
 	GHashTable *pinvoke_scopes;
-
-	/* Maps malloc-ed char* pinvoke scope -> malloced-ed char* filename */
-	GHashTable *pinvoke_scope_filenames;
+#endif
 
 	/* Indexed by MonoGenericParam pointers */
 	GHashTable **gshared_types;
@@ -1136,7 +1135,7 @@ mono_image_set_description (MonoImageSet *);
 MonoImageSet *
 mono_find_image_set_owner (void *ptr);
 
-MONO_API void
+void
 mono_loader_register_module (const char *name, MonoDl *module);
 
 gboolean
