@@ -1234,4 +1234,10 @@ sgen_tarjan_bridge_init (SgenBridgeProcessor *collector)
 	bridge_processor = collector;
 }
 
+#else
+
+#include <mono/utils/mono-compiler.h>
+
+MONO_EMPTY_SOURCE_FILE (sgen_tarjan_bridge);
+
 #endif
