@@ -57,4 +57,8 @@ mono_w32process_get_modules (pid_t pid)
 	return g_slist_reverse (ret);
 }
 
+#else
+
+MONO_EMPTY_SOURCE_FILE (w32process_unix_haiku);
+
 #endif
