@@ -41,6 +41,7 @@ typedef enum {
 	LLVM_ATTR_NO_UNWIND,
 	LLVM_ATTR_NO_INLINE,
 	LLVM_ATTR_OPTIMIZE_FOR_SIZE,
+	LLVM_ATTR_OPTIMIZE_NONE,
 	LLVM_ATTR_IN_REG,
 	LLVM_ATTR_STRUCT_RET,
 	LLVM_ATTR_NO_ALIAS,
@@ -150,6 +151,9 @@ G_EXTERN_C _Unwind_Reason_Code mono_debug_personality (int a, _Unwind_Action b,
 void*
 mono_llvm_create_di_builder (LLVMModuleRef module);
 
+gboolean
+mono_llvm_can_be_gep (LLVMValueRef base, LLVMValueRef* actual_base, LLVMValueRef* actual_offset);
+
 void*
 mono_llvm_di_create_function (void *di_builder, void *cu, LLVMValueRef func, const char *name, const char *mangled_name, const char *dir, const char *file, int line);
 
@@ -174,6 +178,9 @@ mono_llvm_di_set_location (LLVMBuilderRef builder, void *loc_md);
 LLVMValueRef
 mono_llvm_get_or_insert_gc_safepoint_poll (LLVMModuleRef module);
 
+gboolean
+mono_llvm_remove_gc_safepoint_poll (LLVMModuleRef module);
+
 typedef struct {
 	const char* alias;
 	guint32 flag;
