@@ -2,6 +2,7 @@
 #include "mini-runtime.h"
 #include <mono/metadata/mono-debug.h>
 #include <mono/metadata/assembly.h>
+#include <mono/metadata/assembly-internals.h>
 #include <mono/metadata/metadata.h>
 #include <mono/metadata/seq-points-data.h>
 #include <mono/mini/aot-runtime.h>
@@ -15,6 +16,7 @@
 
 #include <emscripten.h>
 
+#include "mono/metadata/assembly-internals.h"
 
 static int log_level = 1;
 
@@ -406,14 +408,16 @@ mono_wasm_set_breakpoint (const char *assembly_name, int method_token, int il_of
 	//resolve the assembly
 	MonoImageOpenStatus status;
 	MonoAssemblyName* aname = mono_assembly_name_new (lookup_name);
-	MonoAssembly *assembly = mono_assembly_load (aname, NULL, &status);
+	MonoAssemblyByNameRequest byname_req;
+	mono_assembly_request_prepare_byname (&byname_req, MONO_ASMCTX_DEFAULT, mono_domain_default_alc (mono_get_root_domain ()));
+	MonoAssembly *assembly = mono_assembly_request_byname (aname, &byname_req, &status);
 	g_free (lookup_name);
 	if (!assembly) {
 		DEBUG_PRINTF (1, "Could not resolve assembly %s\n", assembly_name);
 		return -1;
 	}
 
-	mono_assembly_name_free (aname);
+	mono_assembly_name_free_internal (aname);
 
 	MonoMethod *method = mono_get_method_checked (assembly->image, MONO_TOKEN_METHOD_DEF | method_token, NULL, NULL, error);
 	if (!method) {
@@ -717,7 +721,7 @@ describe_object_properties (guint64 objectId, gboolean isAsyncLocalThis)
 			mono_wasm_add_properties_var(p->name); 
 			sig = mono_method_signature_internal (p->get);
 			res = mono_runtime_try_invoke (p->get, obj, NULL, &exc, error);
-			if (!mono_error_ok (error) && exc == NULL)
+			if (!is_ok (error) && exc == NULL)
 				exc = (MonoObject*) mono_error_convert_to_exception (error);
 			if (exc)
 				describe_value (mono_get_object_type (), &exc);
