@@ -26,6 +26,15 @@ METHOD(op_Explicit)
 METHOD(op_Inequality)
 METHOD(op_Multiply)
 METHOD(op_Subtraction)
+// Vector
+METHOD(ConvertToInt32)
+METHOD(ConvertToUInt32)
+METHOD(ConvertToInt64)
+METHOD(ConvertToUInt64)
+METHOD(ConvertToSingle)
+METHOD(ConvertToDouble)
+METHOD(Narrow)
+METHOD(Widen)
 // BMI1
 METHOD(AndNot)
 METHOD(BitFieldExtract)
