@@ -22,6 +22,7 @@
 #include "mono/sgen/sgen-memory-governor.h"
 #include "mono/sgen/sgen-workers.h"
 #include "mono/sgen/sgen-client.h"
+#include "mono/utils/memfuncs.h"
 
 /*
  * The allowance we are targeting is a third of the current heap size. Still, we
@@ -73,7 +74,6 @@ static mword last_used_slots_size = 0;
 
 static mword sgen_memgov_available_free_space (void);
 
-
 /* GC trigger heuristics. */
 
 static void
@@ -235,6 +235,11 @@ sgen_memgov_major_post_sweep (mword used_slots_size)
 
 		sgen_add_log_entry (log_entry);
 	}
+
+	sgen_gc_info.heap_size_bytes = sgen_major_collector.get_num_major_sections () * sgen_major_collector.section_size + sgen_los_memory_usage_total;
+	sgen_gc_info.fragmented_bytes = sgen_gc_info.heap_size_bytes - sgen_los_memory_usage - (used_slots_size + sgen_total_allocated_major - total_allocated_major_end);
+	sgen_gc_info.memory_load_bytes = sgen_los_memory_usage + used_slots_size + sgen_total_allocated_major - total_allocated_major_end;
+
 	last_used_slots_size = used_slots_size;
 }
 
@@ -244,6 +249,8 @@ sgen_memgov_major_collection_start (gboolean concurrent, const char *reason)
 	need_calculate_minor_collection_allowance = TRUE;
 	major_start_heap_size = get_heap_size ();
 
+	sgen_gc_info.high_memory_load_threshold_bytes = major_collection_trigger_size;
+
 	if (debug_print_allowance) {
 		SGEN_LOG (0, "Starting collection with heap size %ld bytes", (long)major_start_heap_size);
 	}
@@ -493,8 +500,15 @@ sgen_memgov_init (size_t max_heap, size_t soft_limit, gboolean debug_allowance,
 
 	sgen_register_fixed_internal_mem_type (INTERNAL_MEM_LOG_ENTRY, sizeof (SgenLogEntry));
 
-	if (max_heap == 0)
+	if (max_heap == 0) {
+		sgen_gc_info.total_available_memory_bytes = mono_determine_physical_ram_size ();
+
+		if (!sgen_gc_info.total_available_memory_bytes){
+			SGEN_LOG(9, "Warning: Unable to determine physical ram size for GCMemoryInfo");
+		}
+
 		return;
+	}
 
 	if (max_heap < soft_limit) {
 		sgen_env_var_error (MONO_GC_PARAMS_NAME, "Setting to minimum.", "`max-heap-size` must be at least as large as `soft-heap-limit`.");
@@ -505,7 +519,9 @@ sgen_memgov_init (size_t max_heap, size_t soft_limit, gboolean debug_allowance,
 		sgen_env_var_error (MONO_GC_PARAMS_NAME, "Setting to minimum.", "`max-heap-size` must be at least 4 times as large as `nursery size`.");
 		max_heap = SGEN_DEFAULT_NURSERY_SIZE * 4;
 	}
-	max_heap_size = max_heap - SGEN_DEFAULT_NURSERY_SIZE;
+	max_heap_size = max_heap;
+
+	sgen_gc_info.total_available_memory_bytes = max_heap;
 
 	if (allowance_ratio)
 		default_allowance_nursery_size_ratio = allowance_ratio;
