@@ -28,6 +28,22 @@
 #include <stddef.h>   /* for size_t */
 #include <mono/utils/mono-compiler.h>
 
+// Returning an undefined struct by value from extern "C"
+// is sometimes an error. Mono does not use the code (dlmallinfo)..
+//
+// Other fixes:
+//   - define the struct
+//   - #if __cplusplus
+//   - #if HOST_WASM
+//   - #ifndef HAVE_USR_INCLUDE_MALLOC_H
+//   - Make it not extern "C".
+//   - Return the struct through an out parameter.
+//   - remove extern "C" entirely in dlmalloc.
+//
+// Mono does not use the function (dlmallinfo).
+//
+#define NO_MALLINFO 1 /* mono */
+
 #ifdef __cplusplus
 extern "C" {
 #endif
