@@ -145,18 +145,57 @@ static const char *configured_timeline_dir;
 static MonoSummaryTimeline log;
 
 static void
-file_for_summary_stage (const char *directory, MonoSummaryStage stage, gchar *buff, size_t sizeof_buff)
+file_for_stage_breadcrumb (const char *directory, MonoSummaryStage stage, gchar *buff, size_t sizeof_buff)
 {
 	g_snprintf (buff, sizeof_buff, "%s%scrash_stage_%d", directory, G_DIR_SEPARATOR_S, stage);
 }
 
 static void
-create_stage_mark_file (void)
+file_for_dump_reason_breadcrumb (const char *directory, const char *dump_reason, gchar *buff, size_t sizeof_buff)
+{
+	g_snprintf (buff, sizeof_buff, "%s%scrash_reason_%s", directory, G_DIR_SEPARATOR_S, dump_reason);
+}
+
+static void
+file_for_hash_breadcrumb (const char *directory, MonoStackHash hashes, gchar *buff, size_t sizeof_buff)
+{
+	g_snprintf (buff, sizeof_buff, "%s%scrash_hash_0x%" PRIx64 "", directory, G_DIR_SEPARATOR_S, (uint64_t)hashes.offset_rich_hash);
+}
+
+static void create_breadcrumb (const char *path)
+{
+	int handle = g_open (path, O_WRONLY | O_CREAT, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
+	if (handle < 0) {
+		g_async_safe_printf ("Failed to create breadcrumb file %s\n", path);
+		return;
+	}
+
+	if (close(handle) < 0)
+		g_async_safe_printf ("Failed to close breadcrumb file %s\n", path);
+}
+
+static void
+create_stage_breadcrumb (void)
 {
 	char out_file [200];
-	file_for_summary_stage (log.directory, log.level, out_file, sizeof(out_file));
-	int handle = g_open (out_file, O_WRONLY | O_CREAT, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
-	close(handle);
+	file_for_stage_breadcrumb (log.directory, log.level, out_file, sizeof(out_file));
+	create_breadcrumb (out_file);
+}
+
+static void
+create_dump_reason_breadcrumb (const char *dump_reason)
+{
+	char out_file [200];
+	file_for_dump_reason_breadcrumb (log.directory, dump_reason, out_file, sizeof(out_file));
+	create_breadcrumb (out_file);
+}
+
+void
+mono_create_crash_hash_breadcrumb (MonoThreadSummary *thread)
+{
+	char out_file [200];
+	file_for_hash_breadcrumb (log.directory, thread->hashes, out_file, sizeof(out_file));
+	create_breadcrumb (out_file);
 }
 
 gboolean
@@ -172,7 +211,7 @@ mono_summarize_set_timeline_dir (const char *directory)
 }
 
 void
-mono_summarize_timeline_start (void)
+mono_summarize_timeline_start (const char *dump_reason)
 {
 	memset (&log, 0, sizeof (log));
 
@@ -180,6 +219,7 @@ mono_summarize_timeline_start (void)
 		return;
 
 	log.directory = configured_timeline_dir;
+	create_dump_reason_breadcrumb (dump_reason);
 	mono_summarize_timeline_phase_log (MonoSummarySetup);
 }
 
@@ -246,7 +286,7 @@ mono_summarize_timeline_phase_log (MonoSummaryStage next)
 	g_assertf(out_level == next || next == MonoSummaryDoubleFault, "Log Error: Log transition to %d, actual expected next step is %d\n", next, out_level);
 
 	log.level = out_level;
-	create_stage_mark_file ();
+	create_stage_breadcrumb ();
 	// To check, comment out normally
 	// DO NOT MERGE UNCOMMENTED
 	// As this does a lot of FILE io
@@ -323,7 +363,7 @@ static gboolean
 timeline_has_level (const char *directory, char *log_file, size_t log_file_size, gboolean clear, MonoSummaryStage stage)
 {
 	memset (log_file, 0, log_file_size);
-	file_for_summary_stage (directory, stage, log_file, log_file_size);
+	file_for_stage_breadcrumb (directory, stage, log_file, log_file_size);
 	gboolean exists = g_file_test (log_file, G_FILE_TEST_EXISTS);
 	if (clear && exists) 
 		remove (log_file);
@@ -1001,6 +1041,26 @@ mono_native_state_add_process_map (MonoStateWriter *writer)
 
 #endif
 
+static void
+mono_native_state_add_logged_message (MonoStateWriter *writer, const char *object_key, const char *msg)
+{
+	if (msg != NULL) {
+		assert_has_space (writer);
+		mono_state_writer_indent (writer);
+		mono_state_writer_object_key (writer, object_key);
+
+		size_t length;
+		const char *pos;
+		if ((pos = strchr (msg, '\n')) != NULL)
+			length = (size_t)(pos - msg);
+		else
+			length = strlen (msg);
+		length = MIN (length, INT_MAX);
+
+		mono_state_writer_printf(writer, "\"%.*s\",\n", (int)length, msg);
+	}
+}
+
 static void
 mono_native_state_add_prologue (MonoStateWriter *writer)
 {
@@ -1019,21 +1079,11 @@ mono_native_state_add_prologue (MonoStateWriter *writer)
 	mono_native_state_add_memory (writer);
 
 	const char *assertion_msg = g_get_assertion_message ();
-	if (assertion_msg != NULL) {
-		assert_has_space (writer);
-		mono_state_writer_indent (writer);
-		mono_state_writer_object_key (writer, "assertion_message");
+	mono_native_state_add_logged_message (writer, "assertion_message", assertion_msg);
 
-		size_t length;
-		const char *pos;
-		if ((pos = strchr (assertion_msg, '\n')) != NULL)
-			length = (size_t)(pos - assertion_msg);
-		else
-			length = strlen (assertion_msg);
-		length = MIN (length, INT_MAX);
-
-		mono_state_writer_printf(writer, "\"%.*s\",\n", (int)length, assertion_msg);
-	}
+	const char *failfast_msg = mono_crash_get_failfast_msg ();
+	mono_native_state_add_logged_message (writer, "failfast_message", failfast_msg);
+	
 
 #ifndef MONO_PRIVATE_CRASHES
 	mono_native_state_add_process_map (writer);
@@ -1168,3 +1218,23 @@ mono_dump_complete (void)
 {
 	return (mono_atomic_xchg_i32(&dump_status, 0) == 1);  // return true if we completed the dump
 }
+
+static char *saved_failfast_msg;
+
+/**
+ * mono_crash_save_failfast_msg:
+ * \param msg the message to save.  Takes ownership, caller shouldn't free
+ *
+ * \returns the previous message - caller is responsible for freeing.
+ */
+char*
+mono_crash_save_failfast_msg (char *msg)
+{
+	return (char*) mono_atomic_xchg_ptr ((gpointer*)&saved_failfast_msg, (void*)msg);
+}
+
+const char*
+mono_crash_get_failfast_msg (void)
+{
+	return saved_failfast_msg;
+}
